/* gtkpadspring.c: An example of a custom spring
 * 
 * Copyright (c) 2005 Red Hat, Inc.
 * Author Matthias Clasen
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "gtkspringlayout.h"
#include "gtkpadspring.h"


static GType gtk_pad_spring_get_type (void) G_GNUC_CONST;
#define GTK_TYPE_PAD_SPRING (gtk_pad_spring_get_type ())
#define GTK_PAD_SPRING(object) \
  (G_TYPE_CHECK_INSTANCE_CAST ((object), GTK_TYPE_PAD_SPRING, GtkPadSpring))

typedef struct _GtkPadSpring GtkPadSpring;
typedef GtkSpringClass GtkPadSpringClass;

struct _GtkPadSpring 
{
  GtkSpring spring;

  GtkSpring *spring1;
  GtkSpring *spring2;
  GtkSpring *spring3;
};

G_DEFINE_TYPE(GtkPadSpring, gtk_pad_spring, GTK_TYPE_SPRING)

static gint 
gtk_pad_spring_get_minimum (GtkSpring *spring)
{
  if (spring->min == GTK_SPRING_UNSET)
    {
      GtkPadSpring *s = GTK_PAD_SPRING (spring);

      gint v1, v2, v3;
      
      v1 = gtk_spring_get_minimum (s->spring1);
      v2 = gtk_spring_get_minimum (s->spring2);
      v3 = gtk_spring_get_minimum (s->spring3);

      spring->min = v1 + v2 + v3;
    }
  
  return spring->min;
}

static gint 
gtk_pad_spring_get_preferred (GtkSpring *spring)
{
  if (spring->preferred == GTK_SPRING_UNSET)
    {
      GtkPadSpring *s = GTK_PAD_SPRING (spring);

      gint v1, v2, v3;
      
      v1 = gtk_spring_get_preferred (s->spring1);
      v2 = gtk_spring_get_preferred (s->spring2);
      v3 = gtk_spring_get_preferred (s->spring3);

      spring->preferred = v1 + v2 + v3;
    }
  
  return spring->preferred;
}

static gint 
gtk_pad_spring_get_maximum (GtkSpring *spring)
{
  if (spring->max == GTK_SPRING_UNSET)
    {
      GtkPadSpring *s = GTK_PAD_SPRING (spring);

      gint v1, v2, v3;
      
      v1 = gtk_spring_get_maximum (s->spring1);
      v2 = gtk_spring_get_maximum (s->spring2);
      v3 = gtk_spring_get_maximum (s->spring3);

      spring->max = v1 + v2 + v3;
    }
  
  return spring->max;
}

static gint 
gtk_pad_spring_get_value (GtkSpring *spring)
{
  if (spring->value == GTK_SPRING_UNSET)
    {
      GtkPadSpring *s = GTK_PAD_SPRING (spring);

      gint v1, v2, v3;
      
      v1 = gtk_spring_get_value (s->spring1);
      v2 = gtk_spring_get_value (s->spring2);
      v3 = gtk_spring_get_value (s->spring3);

      spring->value = v1 + v2 + v3;
    }
  
  return spring->value;
}


static void
gtk_pad_spring_set_value (GtkSpring *spring,
			  gint       value)
{
  GtkPadSpring *s = GTK_PAD_SPRING (spring);
  gint value1, value2, value3;
  gint pref1, pref2, pref3;
  gint min1, min2, min3;
  gint max1, max2, max3;
  gint extra, extra1, extra3;
  gint stretch, stretch1, stretch3;

  if (value == GTK_SPRING_UNSET)
    {
      if (spring->value != GTK_SPRING_UNSET)
	{
	  spring->min = GTK_SPRING_UNSET;
	  spring->preferred = GTK_SPRING_UNSET;
	  spring->max = GTK_SPRING_UNSET;
	  
	  spring->value = GTK_SPRING_UNSET;
	  gtk_spring_set_value (s->spring1, GTK_SPRING_UNSET);
	  gtk_spring_set_value (s->spring2, GTK_SPRING_UNSET);
	  gtk_spring_set_value (s->spring3, GTK_SPRING_UNSET);
	}

      return;
    }

  spring->value = value;
  
  min1 = gtk_spring_get_minimum (s->spring1);
  min2 = gtk_spring_get_minimum (s->spring2);
  min3 = gtk_spring_get_minimum (s->spring3);
  max1 = gtk_spring_get_maximum (s->spring1);
  max2 = gtk_spring_get_maximum (s->spring2);
  max3 = gtk_spring_get_maximum (s->spring3);
  pref2 = gtk_spring_get_preferred (s->spring2);
 
  if (value <= min1 + min2 + min3)
    value2 = min2;
  else if (value <= min1 + pref2 + min3)
    value2 = value - min1 - min3;
  else if (value <= max1 + pref2 + max3)
    value2 = pref2;
  else if (value <= max1 + max2 + max3)
    value2 = value - max1 - max3;
  else 
    value2 = max2;

  gtk_spring_set_value (s->spring2, value2);
  
  if (value == value2)
    {
      value1 = 0;
      value3 = 0;
    }
  else
    {
      pref1 = gtk_spring_get_preferred (s->spring1);
      pref3 = gtk_spring_get_preferred (s->spring3);
      extra = value - value2 - pref1 - pref3;
      
      if (extra == 0) 
	{
	  extra1 = 0;
	  extra3 = 0;
	} 
      else 
	{
	  if (extra > 0) 
	    {
	      stretch1 = gtk_spring_get_maximum (s->spring1) - pref1; 
	      stretch3 = gtk_spring_get_maximum (s->spring3) - pref3; 
	    } 
	  else 
	    {
	      stretch1 = pref1 - gtk_spring_get_minimum (s->spring1);
	      stretch3 = pref3 - gtk_spring_get_minimum (s->spring3);
	    } 
	  
	  stretch = stretch1 + stretch3;
	  if (stretch == 0) 
	    {
	      extra1 = extra/2;
	      extra3 = extra - extra1;
	    } 
	  else if (stretch1 > stretch3) 
	    {
	      extra1 = extra * stretch1 / stretch;
	      extra3 = extra - extra1;
	    } 
	  else 
	    {
	      extra3 = extra * stretch3 / stretch;
	      extra1 = extra - extra3;
	    }
	}
      
      value1 = pref1 + extra1;
      value3 = pref3 + extra3;
    }
	    
  gtk_spring_set_value (s->spring1, value1);
  gtk_spring_set_value (s->spring2, value2);
  gtk_spring_set_value (s->spring3, value3);
}

static void
gtk_pad_spring_finalize (GObject *object)
{
  GtkPadSpring *s = GTK_PAD_SPRING (object);

  g_object_unref (s->spring1);
  g_object_unref (s->spring2);
  g_object_unref (s->spring3);
}

static gboolean 
gtk_pad_spring_is_cyclic (GtkSpring *spring)
{
  GtkPadSpring *s = GTK_PAD_SPRING (spring);

  return gtk_spring_is_cyclic (s->spring1) ||
    gtk_spring_is_cyclic (s->spring2) ||
    gtk_spring_is_cyclic (s->spring3);
}

static void 
gtk_pad_spring_reset_cyclic (GtkSpring *spring)
{
  GtkPadSpring *s = GTK_PAD_SPRING (spring);

  gtk_spring_reset_cyclic (s->spring1);
  gtk_spring_reset_cyclic (s->spring2);
  gtk_spring_reset_cyclic (s->spring3);
}

static void
gtk_pad_spring_class_init (GtkPadSpringClass *class)
{
  GObjectClass *object_class = G_OBJECT_CLASS (class);
  GtkSpringClass *spring_class = GTK_SPRING_CLASS (class);
  
  object_class->finalize = gtk_pad_spring_finalize;

  spring_class->get_minimum_value = gtk_pad_spring_get_minimum;
  spring_class->get_preferred_value = gtk_pad_spring_get_preferred;
  spring_class->get_maximum_value = gtk_pad_spring_get_maximum;
  spring_class->get_value = gtk_pad_spring_get_value;
  spring_class->set_value = gtk_pad_spring_set_value;
  spring_class->is_cyclic = gtk_pad_spring_is_cyclic;
  spring_class->reset_cyclic = gtk_pad_spring_reset_cyclic;
}

static void
gtk_pad_spring_init (GtkPadSpring *spring)
{
}

GtkSpring *
gtk_spring_pad (GtkSpring *spring1,
		GtkSpring *spring2,
		GtkSpring *spring3)
{
  GtkPadSpring *spring;

  spring = g_object_new (GTK_TYPE_PAD_SPRING, NULL);

  spring->spring1 = gtk_spring_sink (g_object_ref (spring1));
  spring->spring2 = gtk_spring_sink (g_object_ref (spring2));
  spring->spring3 = gtk_spring_sink (g_object_ref (spring3));

  return GTK_SPRING (spring);
}


