/* gtkspringlayout.h
 * 
 * Copyright (c) 2005 Red Hat, Inc.
 * Author Matthias Clasen
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef __GTK_SPRING_LAYOUT_H__
#define __GTK_SPRING_LAYOUT_H__

#include <gtk/gtk.h>

G_BEGIN_DECLS

#define GTK_TYPE_SPRING_LAYOUT (gtk_spring_layout_get_type ())
#define GTK_SPRING_LAYOUT(object) \
  (G_TYPE_CHECK_INSTANCE_CAST ((object), GTK_TYPE_SPRING_LAYOUT, GtkSpringLayout))
#define GTK_SPRING_LAYOUT_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST ((klass), GTK_TYPE_SPRING_LAYOUT, GtkSpringLayoutClass))
#define GTK_IS_SPRING_LAYOUT(object) \
  (G_TYPE_CHECK_INSTANCE_TYPE ((object), GTK_TYPE_SPRING_LAYOUT))
#define GTK_IS_SPRING_LAYOUT_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_SPRING_LAYOUT))
#define GTK_SPRING_LAYOUT_GET_CLASS(object) \
  (G_TYPE_INSTANCE_GET_CLASS ((object), GTK_TYPE_SPRING_LAYOUT, GtkSpringLayoutClass))


typedef struct _GtkSpringLayout GtkSpringLayout;
typedef struct _GtkSpringLayoutClass GtkSpringLayoutClass;

struct _GtkSpringLayout
{
  /*< private >*/
  GtkContainer parent;

  GList *children;

  gpointer constraints;

  gpointer cycle;
};

struct _GtkSpringLayoutClass
{
  /*< private >*/
  GtkContainerClass parent_class;

  void (*__gtk_reserved1);
  void (*__gtk_reserved2);
  void (*__gtk_reserved3);
  void (*__gtk_reserved4);
  void (*__gtk_reserved5);
  void (*__gtk_reserved6);
  void (*__gtk_reserved7);
  void (*__gtk_reserved8);
};

typedef struct _GtkSpring GtkSpring;

GType      gtk_spring_layout_get_type         (void) G_GNUC_CONST;
GtkWidget *gtk_spring_layout_new              (void);

typedef enum {
  GTK_CONSTRAINT_LEFT,
  GTK_CONSTRAINT_X = GTK_CONSTRAINT_LEFT,
  GTK_CONSTRAINT_TOP,
  GTK_CONSTRAINT_Y = GTK_CONSTRAINT_TOP,
  GTK_CONSTRAINT_RIGHT,
  GTK_CONSTRAINT_BOTTOM,
  GTK_CONSTRAINT_WIDTH,
  GTK_CONSTRAINT_HEIGHT
} GtkConstraintType;

void       gtk_spring_layout_add_spring       (GtkSpringLayout   *layout,
					       GtkWidget         *child,
					       GtkConstraintType  pos,
					       gint               min,
					       gint               preferred,
					       gint               max,
					       GtkWidget         *target,
					       GtkConstraintType  target_pos);
void       gtk_spring_layout_add_strut        (GtkSpringLayout   *layout,
					       GtkWidget         *widget,
					       GtkConstraintType  pos,
					       gint               size,
					       GtkWidget         *target,
					       GtkConstraintType  target_pos);
void       gtk_spring_layout_set_constraint   (GtkSpringLayout   *layout,
					       GtkWidget         *widget,
					       GtkConstraintType  pos,
					       GtkSpring         *spring);
GtkSpring *gtk_spring_layout_get_constraint   (GtkSpringLayout   *layout,
					       GtkWidget         *widget,
					       GtkConstraintType  pos);

void       gtk_spring_layout_dump             (GtkSpringLayout   *layout);


#define GTK_TYPE_SPRING \
  (gtk_spring_get_type ())
#define GTK_SPRING(object) \
  (G_TYPE_CHECK_INSTANCE_CAST ((object), GTK_TYPE_SPRING, GtkSpring))
#define GTK_SPRING_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST ((klass), GTK_TYPE_SPRING, GtkSpringClass))
#define GTK_IS_SPRING(object) \
  (G_TYPE_CHECK_INSTANCE_TYPE ((object), GTK_TYPE_SPRING))
#define GTK_IS_SPRING_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_SPRING))
#define GTK_SPRING_GET_CLASS(object) \
  (G_TYPE_INSTANCE_GET_CLASS ((object), GTK_TYPE_SPRING, GtkSpringClass))

#define GTK_SPRING_UNSET G_MININT

typedef struct _GtkSpringPrivate GtkSpringPrivate;
struct _GtkSpring
{
  GObject object;

  /*< protected >*/
  gint min;
  gint max;
  gint preferred;
  gint value;  

  /*< private >*/
  gint cyclic;
  gboolean floating;
};

typedef struct _GtkSpringClass GtkSpringClass;
struct _GtkSpringClass
{
  GObjectClass parent_class;
  
  gint     (*get_minimum_value)   (GtkSpring *spring);
  gint     (*get_preferred_value) (GtkSpring *spring);
  gint     (*get_maximum_value)   (GtkSpring *spring);
  gint     (*get_value)           (GtkSpring *spring);
  void     (*set_value)           (GtkSpring *spring, 
				   gint       value);
  gboolean (*is_cyclic)           (GtkSpring *spring);
  void     (*reset_cyclic)        (GtkSpring *spring);

  void (*__gtk_reserved1);
  void (*__gtk_reserved2);
  void (*__gtk_reserved3);
  void (*__gtk_reserved4);
};

GType      gtk_spring_get_type      (void) G_GNUC_CONST;
GtkSpring *gtk_spring_sink          (GtkSpring *spring);
GtkSpring *gtk_spring_simple        (gint       min, 
				     gint       preferred, 
				     gint       max);
GtkSpring *gtk_spring_constant      (gint       c);
GtkSpring *gtk_spring_minus         (GtkSpring *spring1);
GtkSpring *gtk_spring_sum           (GtkSpring *spring1, 
				     GtkSpring *spring2);
GtkSpring *gtk_spring_scale         (GtkSpring *spring1, 
				     gdouble    factor);
GtkSpring *gtk_spring_difference    (GtkSpring *spring1, 
				     GtkSpring *spring2);
GtkSpring *gtk_spring_max           (GtkSpring *spring1, 
				     GtkSpring *spring2);
GtkSpring *gtk_spring_width         (GtkWidget *widget);
GtkSpring *gtk_spring_height        (GtkWidget *widget);
GtkSpring *gtk_spring_proxy         (GtkWidget *widget,
				     GtkConstraintType pos);

gint       gtk_spring_get_minimum   (GtkSpring *spring);
gint       gtk_spring_get_preferred (GtkSpring *spring);
gint       gtk_spring_get_maximum   (GtkSpring *spring);
gint       gtk_spring_get_value     (GtkSpring *spring);
void       gtk_spring_set_value     (GtkSpring *spring,
				     gint       value);
gboolean   gtk_spring_is_cyclic     (GtkSpring *spring);
void       gtk_spring_reset_cyclic  (GtkSpring *spring);

G_END_DECLS

#endif  /* __GTK_SPRING_LAYOUT_H__ */
