/* testsprings.c
 * 
 * Copyright (c) 2005 Red Hat, Inc.
 * Author Matthias Clasen
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include "gtkspringlayout.h"
#include "gtkpadspring.h"

/*
 * Simple constraints 
 */
static GtkWidget *
test1 (void)
{
  GtkWidget *layout;  
  GtkWidget *button1;
  GtkWidget *button2;

  layout = gtk_spring_layout_new ();

  button1 = gtk_button_new_with_label ("Button1");
  button2 = gtk_button_new_with_label ("Button2");

  gtk_container_add (GTK_CONTAINER (layout), button1);
  gtk_container_add (GTK_CONTAINER (layout), button2);

  gtk_spring_layout_add_spring (GTK_SPRING_LAYOUT (layout),
                                button2, GTK_CONSTRAINT_TOP,
                                20, 50,  100,
                                button1, GTK_CONSTRAINT_BOTTOM);

  gtk_spring_layout_add_spring (GTK_SPRING_LAYOUT (layout),
                                button2, GTK_CONSTRAINT_LEFT,
                                10, 20, 30,
                                button1, GTK_CONSTRAINT_LEFT);

  /*gtk_spring_layout_dump (layout);*/

  return layout;
}

/*
 * Flexible children with extra-width
 */
static GtkWidget *
test2 (void)
{
  GtkWidget *layout;
  GtkWidget *label;
  GtkWidget *entry;

  layout = gtk_spring_layout_new ();

  label = gtk_label_new ("Label");
  entry = gtk_entry_new ();

  gtk_widget_set_name (label, "label");
  gtk_widget_set_name (entry, "entry");

  gtk_container_add (GTK_CONTAINER (layout), label);
  gtk_container_add (GTK_CONTAINER (layout), entry);

  gtk_spring_layout_add_strut (GTK_SPRING_LAYOUT (layout), 
			       label, GTK_CONSTRAINT_TOP,
			       5,
			       layout, GTK_CONSTRAINT_TOP);

  gtk_spring_layout_add_strut (GTK_SPRING_LAYOUT (layout), 
			       label, GTK_CONSTRAINT_LEFT,
			       5,
			       layout, GTK_CONSTRAINT_LEFT);

  gtk_spring_layout_add_strut (GTK_SPRING_LAYOUT (layout), 
			       entry, GTK_CONSTRAINT_LEFT,
			       5,
			       label, GTK_CONSTRAINT_RIGHT);

  gtk_spring_layout_add_strut (GTK_SPRING_LAYOUT (layout), 
			       entry, GTK_CONSTRAINT_TOP,
			       5,
			       layout, GTK_CONSTRAINT_TOP);

  gtk_spring_layout_add_spring (GTK_SPRING_LAYOUT (layout), 
				layout, GTK_CONSTRAINT_BOTTOM,
				5, 5, 200,
				entry, GTK_CONSTRAINT_BOTTOM);

  gtk_spring_layout_add_spring (GTK_SPRING_LAYOUT (layout), 
			       layout, GTK_CONSTRAINT_RIGHT,
				5, 5, 200,
			       entry, GTK_CONSTRAINT_RIGHT);

  gtk_container_child_set (GTK_CONTAINER (layout), entry,
			   "extra-width", 0,
			   "max-extra-width", 100,
			   NULL);

  return layout;
}

static void
make_compact_grid (GtkSpringLayout *layout,
		   gint             rows, 
		   gint             cols,
		   gint             x_offset,
		   gint             y_offset,
		   gint             x_pad,
		   gint             y_pad)
{
  GtkSpring *x, *y, *width, *height, *xpad, *ypad;
  GList *children, *child;
  gint i, j;

  xpad = gtk_spring_constant (x_pad);
  ypad = gtk_spring_constant (y_pad);
  children = gtk_container_get_children (GTK_CONTAINER (layout));
  child = children;
  x = gtk_spring_constant (x_offset);
  for (i = 0; i < cols; i++) 
    {
      width = gtk_spring_constant (0);
      for (j = 0; j < rows; j++) 
	{
	  child = g_list_nth (children, j * cols + i);
	  width = gtk_spring_max (width, gtk_spring_width (child->data));
	}
      for (j = 0; j < rows; j++) 
	{
	  child = g_list_nth (children, j * cols + i);
	  gtk_spring_layout_set_constraint (layout,
					    child->data,
					    GTK_CONSTRAINT_X, x);
	  gtk_spring_layout_set_constraint (layout,
					    child->data,
					    GTK_CONSTRAINT_WIDTH, width);
        }
      x = gtk_spring_sum (x, gtk_spring_sum (width, xpad));
    }
      
  y = gtk_spring_constant (y_offset);
  for (i = 0; i < rows; i++) 
    {
      height = gtk_spring_constant (0);
      for (j = 0; j < cols; j++) 
	{
	  child = g_list_nth (children, i * cols + j);
	  height = gtk_spring_max (height,
				   gtk_spring_height (child->data));
	}
      for (j = 0; j < cols; j++) 
	{
	  child = g_list_nth (children, i * cols + j);
	  gtk_spring_layout_set_constraint (layout,
					    child->data,
					    GTK_CONSTRAINT_Y, y);
	  gtk_spring_layout_set_constraint (layout,
					    child->data,
					    GTK_CONSTRAINT_HEIGHT, height);
        }
      y = gtk_spring_sum (y, gtk_spring_sum (height, ypad));
    }
  
  gtk_spring_layout_set_constraint (layout, GTK_WIDGET (layout),
				    GTK_CONSTRAINT_BOTTOM, y);
  gtk_spring_layout_set_constraint (layout, GTK_WIDGET (layout),
				    GTK_CONSTRAINT_RIGHT, x);
}

/*
 * A grid
 */
static GtkWidget *
test3 (void)
{
  GtkWidget *layout;
  GtkWidget *label;
  GtkWidget *entry;
  gchar *labels[] = {"Name:", "Fax:", "Email:", "Address:", NULL};
  gint i;

  layout = gtk_spring_layout_new ();

  for (i = 0; labels[i]; i++)
    {
      GtkWidget *label;

      label = gtk_label_new (labels[i]);
      gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
      gtk_container_add (GTK_CONTAINER (layout), label);
      gtk_container_add (GTK_CONTAINER (layout), gtk_entry_new ());
    }
  
  make_compact_grid (GTK_SPRING_LAYOUT (layout), 4, 2, 6, 6, 6, 6);

  return layout;
}

/*
 * Centering with scale springs 
 */
static GtkWidget *
test4 (void)
{
  GtkWidget *layout;
  GtkWidget *label, *button;
  GtkSpring *s1, *s2, *s3;

  layout = gtk_spring_layout_new ();

  label = gtk_label_new ("Short");
  button = gtk_button_new_with_label ("Long text here");

  gtk_container_add (GTK_CONTAINER (layout), label);
  gtk_container_add (GTK_CONTAINER (layout), button);

  gtk_spring_layout_add_strut (GTK_SPRING_LAYOUT (layout), 
			       label, GTK_CONSTRAINT_TOP,
			       10,
			       layout, GTK_CONSTRAINT_TOP);

  gtk_spring_layout_add_strut (GTK_SPRING_LAYOUT (layout), 
			       button, GTK_CONSTRAINT_TOP,
			       10,
			       label, GTK_CONSTRAINT_BOTTOM);

  gtk_spring_layout_add_spring (GTK_SPRING_LAYOUT (layout), 
				layout, GTK_CONSTRAINT_BOTTOM,
				10, 10, 100,
				button, GTK_CONSTRAINT_BOTTOM);

  gtk_spring_layout_add_spring (GTK_SPRING_LAYOUT (layout), 
				button, GTK_CONSTRAINT_LEFT,
				10, 10, 1000,
				layout, GTK_CONSTRAINT_LEFT);

  gtk_spring_layout_add_spring (GTK_SPRING_LAYOUT (layout), 
				layout, GTK_CONSTRAINT_RIGHT,
				10, 10, 1000,
				button, GTK_CONSTRAINT_RIGHT);

  s1 = gtk_spring_width (button);
  s2 = gtk_spring_width (label);
  s3 = gtk_spring_difference (s1, s2);
  s1 = gtk_spring_scale (s3, 0.5);
  s2 = gtk_spring_layout_get_constraint (GTK_SPRING_LAYOUT (layout), 
					 button, GTK_CONSTRAINT_LEFT);
  s3 = gtk_spring_sum (s1, s2);
  
  gtk_spring_layout_set_constraint (GTK_SPRING_LAYOUT (layout), 
				    label, GTK_CONSTRAINT_LEFT,
				    s3);

  return layout;
}

/*
 * A custom spring for padding
 */
static GtkWidget *
test5 (void)
{
  GtkWidget *layout;
  GtkWidget *button;
  GtkSpring *s, *s1, *s2;

  layout = gtk_spring_layout_new ();
  button = gtk_button_new_with_label ("Button");
  gtk_container_add (GTK_CONTAINER (layout), button);

  gtk_spring_layout_set_constraint (GTK_SPRING_LAYOUT (layout), 
				    button, GTK_CONSTRAINT_LEFT,
				    gtk_spring_simple (10, 10, 100));
  
  gtk_spring_layout_add_spring (GTK_SPRING_LAYOUT (layout), 
				layout, GTK_CONSTRAINT_BOTTOM,
				5, 5, 100,
				button, GTK_CONSTRAINT_BOTTOM);

  s1 = gtk_spring_layout_get_constraint (GTK_SPRING_LAYOUT (layout), 
					 button, GTK_CONSTRAINT_LEFT);
  
  s2 = gtk_spring_layout_get_constraint (GTK_SPRING_LAYOUT (layout), 
					 button, GTK_CONSTRAINT_WIDTH);
  s = gtk_spring_pad (s1, s2, gtk_spring_simple (5, 5, 100));

  gtk_spring_layout_set_constraint (GTK_SPRING_LAYOUT (layout), 
				    layout, GTK_CONSTRAINT_RIGHT,
				    s);

  gtk_container_child_set (GTK_CONTAINER (layout), button,
			   "extra-width", 5,
			   "max-extra-width", 1000,
			   NULL);

  return layout;
}

/* 
 * Forcing the same height and position
 */
static GtkWidget *
test6 (void)
{
  GtkWidget *layout;
  GtkWidget *button1;
  GtkWidget *button2;
  GtkSpring *s;

  layout = gtk_spring_layout_new ();
  button1 = gtk_button_new_with_label ("Button");
  button2 = gtk_button_new_with_label ("Double\nButton");
  gtk_container_add (GTK_CONTAINER (layout), button1);
  gtk_container_add (GTK_CONTAINER (layout), button2);

  gtk_spring_layout_add_spring (GTK_SPRING_LAYOUT (layout), 
				button1, GTK_CONSTRAINT_LEFT,
				10, 10, 100,
				layout, GTK_CONSTRAINT_LEFT);

  gtk_spring_layout_add_strut (GTK_SPRING_LAYOUT (layout), 
			       button2, GTK_CONSTRAINT_LEFT,
			       10,
			       button1, GTK_CONSTRAINT_RIGHT);

  gtk_spring_layout_add_spring (GTK_SPRING_LAYOUT (layout), 
				layout, GTK_CONSTRAINT_RIGHT,
				10, 10, 100,
				button2, GTK_CONSTRAINT_RIGHT);

  gtk_spring_layout_add_spring (GTK_SPRING_LAYOUT (layout), 
				button2, GTK_CONSTRAINT_TOP,
				10, 10, 100,
				layout, GTK_CONSTRAINT_TOP);

  gtk_spring_layout_add_spring (GTK_SPRING_LAYOUT (layout), 
				layout, GTK_CONSTRAINT_BOTTOM,
				10, 10, 100,
				button2, GTK_CONSTRAINT_BOTTOM);

  gtk_spring_layout_set_constraint (GTK_SPRING_LAYOUT (layout), 
				    button1, GTK_CONSTRAINT_HEIGHT,
				    gtk_spring_height (button2));

  s = gtk_spring_layout_get_constraint (GTK_SPRING_LAYOUT (layout), 
					button2, GTK_CONSTRAINT_TOP);
  gtk_spring_layout_set_constraint (GTK_SPRING_LAYOUT (layout), 
				    button1, GTK_CONSTRAINT_TOP,
				    s);

  return layout;
}


int
main (int argc, char *argv[])
{
  GtkWidget *window1;
  GtkWidget *window2;
  GtkWidget *window3;
  GtkWidget *window4;
  GtkWidget *window5;
  GtkWidget *window6;

  gtk_init (&argc, &argv);

  window1 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window1), "test1");
  gtk_container_add (GTK_CONTAINER (window1), test1 ());
  gtk_widget_show_all (window1);

  window2 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window2), "test2");
  gtk_container_add (GTK_CONTAINER (window2), test2 ());
  gtk_widget_show_all (window2);

  window3 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window3), "test3");
  gtk_container_add (GTK_CONTAINER (window3), test3 ());
  gtk_widget_show_all (window3);

  window4 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window4), "test4");
  gtk_container_add (GTK_CONTAINER (window4), test4 ());
  gtk_widget_show_all (window4);

  window5 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window5), "test5");
  gtk_container_add (GTK_CONTAINER (window5), test5 ());
  gtk_widget_show_all (window5);

  window6 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window6), "test6");
  gtk_container_add (GTK_CONTAINER (window6), test6 ());
  gtk_widget_show_all (window6);

  gtk_main ();

  return 0;
}
